@@ -9,6 +9,7 @@ import signal
 import logging
 from pathlib import Path
 
+import ezdxf
 from ezdxf import recover
 from ezdxf.lldxf import const
 from ezdxf.lldxf.validator import is_dxf_file
@@ -353,7 +354,6 @@ class Draw(Command):
             frontend = AllVisibleFrontend(ctx, out)
         else:
             frontend = Frontend(ctx, out)
-
         frontend.draw_layout(layout, finalize=True)
 
         if args.out is not None:
@@ -400,6 +400,12 @@ class View(Command):
             help="set custom line weight scaling, default is 0 to disable "
             "line weights at all",
         )
+        parser.add_argument(
+            "--scale",
+            type=float,
+            default=1.0,
+            help="set set overall scaling factor > 1e-9 to scale the content",
+        )
 
     @staticmethod
     def run(args):
@@ -411,6 +417,9 @@ class View(Command):
             sys.exit(1)
         from ezdxf.addons.drawing.qtviewer import CadViewer
 
+        if args.scale < 1e-9:
+            print("Scaling factor too small or negative!")
+            sys.exit(2)
         signal.signal(signal.SIGINT, signal.SIG_DFL)  # handle Ctrl+C properly
         app = QtWidgets.QApplication(sys.argv)
         set_app_icon(app)
@@ -423,8 +432,12 @@ class View(Command):
         filename = args.file
         if filename:
             doc, auditor = load_document(filename)
-            viewer.set_document(doc, auditor)
-            viewer.draw_layout(args.layout)
+            viewer.set_document(
+                doc,
+                auditor,
+                layout=args.layout,
+                overall_scaling_factor=args.scale,
+            )
         sys.exit(app.exec_())
 
 
